-module (downloadbench).

-export ([main/1]).

replaceActor (Name, _From, _To, _Next, 0) ->
    main ! {done, Name};
replaceActor (Name, From, To, Next, Capacity) ->
    receive
	{replace, Order} ->
	    Replaced = re:replace (Order, From, To),
	    %io:format ("~s~n", [Replaced]),
	    if
		is_pid (Next) ->
		    Next ! {replace, Replaced};
		true ->
		    ok
	    end,
	    replaceActor (Name, From, To, Next, Capacity - 1)
    end.

downloadActor (Capacity, IndexActor) when is_pid (IndexActor) ->
    %io:format ("registering download~n"),
    register (download, self ()),
    replaceActor (download, <<"Requested ">>, <<"Downloaded ">>, IndexActor, Capacity).

indexActor (Capacity, WriteActor) when is_pid (WriteActor) ->
    %io:format ("registering index~n"),
    register (index, self ()),
    replaceActor (index, <<"Downloaded ">>, <<"Indexed ">>, WriteActor, Capacity).

writeActor (Capacity) ->
    %io:format ("registering write~n"),
    register (write, self ()),
    replaceActor (write, <<"Indexed ">>, <<"Wrote ">>, none, Capacity).

send_messages (_DownloadActor, _Capacity, _Capacity) ->
    ok;
send_messages (DownloadActor, It, Capacity) ->
    DownloadActor ! {replace, [<<"Requested ">>, list_to_binary (integer_to_list (It))]},
    send_messages (DownloadActor, It+1, Capacity).

send_messages (DownloadActor, Capacity) ->
    send_messages (DownloadActor, 0, Capacity).

do_run_benchmark (DownloadActor, Capacity) ->
    ok = send_messages (DownloadActor, Capacity),
    receive
	{done, write} ->
	    ok
    end.

run_benchmark (Capacity) ->
    io:format ("downloadbench.erl ~p~n", [Capacity]),

    WriteActor =
	spawn (
	  fun () ->
		  writeActor (Capacity)
	  end),
    IndexActor =
	spawn (
	  fun () ->
		  indexActor (Capacity, WriteActor)
	  end),
    DownloadActor =
	spawn (
	  fun () ->
		  downloadActor (Capacity, IndexActor)
	  end),

    {Time, ok} = timer:tc (fun () -> do_run_benchmark (DownloadActor, Capacity) end),
    TimeMs = Time/1000.0,
    io:format ("Time: ~pms~n", [TimeMs]),
    TimeMs.

main ([Arg]) ->
    register (main, self ()),
    Capacity = list_to_integer (Arg),
    NRuns = 20,
    TotalTime = lists:foldl (
		  fun (_, AccTime) ->
			  Time = run_benchmark (Capacity),
			  AccTime + Time
		  end, 0, lists:seq (1,NRuns)),
    io:format ("Average Time: ~pms~n", [TotalTime/(1.0*NRuns)]),
    erlang:halt ().
